﻿# -*- coding: utf-8 -*-
"""
Модуль обновления проектов и ПО через svn

(c)2013 ЗАО "РИПАС",
www.ripas.ru
"""
from gui import msgError

checkUpdatePeriodSec = 60
noPySvn = True
TortoiseProc = None
kdesvn = None
projectUpdater = None
programUpdater = None

def manualUpdateProject():
    if noPySvn or projectUpdater is None: return
    projectUpdater.checkOnce()


def manualUpdateProgram():
    if noPySvn: return
    programUpdater.checkOnce()


def setAutoUpdateProject(start=True):
    if noPySvn or projectUpdater is None:
        return
    from settings import userSettings
    userSettings.setValue('projectAutoupdate', 1 if start else 0)
    projectUpdater.setAutoUpdate(start)


def setAutoUpdateProgram(start=True):
    if noPySvn or programUpdater is None:
        return
    from settings import userSettings
    userSettings.setValue('programAutoupdate', 1 if start else 0)
    programUpdater.setAutoUpdate(start)


def projectUpdateProc(self, repo_path):
    if len(repo_path) == 0:
        return
    from actions import closeProject, openFile
    from RoadCentre import project
    import subprocess
    from gui import osmView as graphicsView
    keep_zoom = graphicsView.zoomF
    keep_pos = graphicsView.mapToScene(graphicsView.viewport().rect).boundingRect().center()
    prj_dir = repo_path
    prj_file = project.fileName
    if len(prj_file) == 0:
        return
    if closeProject():
        try:
            if TortoiseProc:
                subprocess.call(TortoiseProc + ' /command:update /path:"%s"' % prj_dir)
            elif kdesvn:
                ret_code = subprocess.call([kdesvn, 'exec', 'update', prj_dir])
            else:
                self.client.update(prj_dir)
        except:
            import sys
            msgError(sys.exc_info())
            if self.update_thread.is_alive() and self.sleep_event.isSet():
                self.setAutoUpdate(False)
        graphicsView.setProperty('ignore_default_preset', True)
        openFile(prj_file)
        graphicsView.setProperty('ignore_default_preset', False)
        graphicsView.zoomF = keep_zoom
        graphicsView.centerOn(keep_pos)

    del subprocess

def getSvnClient():
    global TortoiseProc
    import os,sys
    from PythonQt import QtGui
    from settings import userSettings
    if sys.platform[:3] == 'win':
        if TortoiseProc is None:
            TortoiseProc = userSettings.value('TortoiseProc')
        if TortoiseProc is None:
            TortoiseProc = which('TortoiseProc')
        if TortoiseProc and os.path.exists(TortoiseProc):
            return TortoiseProc
        fileName = QtGui.QFileDialog.getOpenFileName(0,u"Выберите файл клиента SVN","","TortoiseProc.exe")
        if fileName and os.path.exists(fileName):
            userSettings.setValue('TortoiseProc',fileName)
        return fileName
    else:
        return kdesvn


def commitProject():
    global TortoiseProc
    if sys.platform[:3] == 'win':
        TortoiseProc = getSvnClient()

    if not TortoiseProc and not kdesvn:
        return
    from RoadCentre import project
    from actions import closeProject, openFile
    import os, subprocess
    from gui import osmView as graphicsView

    prj_file = project.fileName
    if len(prj_file) == 0:
        return
    if kdesvn:
        ret_code = subprocess.call([kdesvn, 'exec', 'commit', os.path.dirname(prj_file)])
    else:
        ret_code = subprocess.call(TortoiseProc + ' /command:commit /path:"%s"' % os.path.dirname(prj_file))
    if ret_code == 0:
        keep_zoom = graphicsView.zoomF
        keep_pos = graphicsView.mapToScene(graphicsView.viewport().rect).boundingRect().center()
        if closeProject():
            graphicsView.setProperty('ignore_default_preset', True)
            openFile(prj_file)
            graphicsView.setProperty('ignore_default_preset', False)
            graphicsView.zoomF = keep_zoom
            graphicsView.centerOn(keep_pos)
        if kdesvn:
            subprocess.call([kdesvn, 'exec', 'update', os.path.dirname(prj_file)])
        else:
            subprocess.call(TortoiseProc + ' /command:update /path:"%s"' % os.path.dirname(prj_file))
    del os, project, subprocess


def revertProject():
    global TortoiseProc
    if sys.platform[:3] == 'win':
        TortoiseProc = getSvnClient()
    if not TortoiseProc and not kdesvn:
        return
    from RoadCentre import project
    from actions import closeProject, openFile
    import os, subprocess
    from gui import osmView as graphicsView
    prj_file = project.fileName
    if len(prj_file) == 0:
        return
    if kdesvn:
        ret_code = subprocess.call([kdesvn, 'exec', 'revert', os.path.dirname(prj_file)])
    else:
        ret_code = subprocess.call(TortoiseProc + ' /command:revert /path:"%s"' % os.path.dirname(prj_file))
    if ret_code == 0:
        keep_zoom = graphicsView.zoomF
        keep_pos = graphicsView.mapToScene(graphicsView.viewport().rect).boundingRect().center()
        if closeProject():
            graphicsView.setProperty('ignore_default_preset', True)
            openFile(prj_file)
            graphicsView.setProperty('ignore_default_preset', False)
            graphicsView.zoomF = keep_zoom
            graphicsView.centerOn(keep_pos)
    del os, project, subprocess


def which(file_name):
    import sys, os
    from PythonQt import QtCore
    sep = ';'
    if sys.platform.startswith('linux'):
        sep = ':'
    path_list = os.environ['PATH'].split(sep)
    path_list.append(QtCore.QCoreApplication.applicationDirPath())
    pf = os.environ.get('ProgramW6432')
    if pf is not None:
        path_list.append(os.path.join(pf,"TortoiseSVN\\bin"))
    pf = os.environ.get('ProgramFiles')
    if pf is not None:
        path_list.append(os.path.join(pf,"TortoiseSVN\\bin"))
    for p in path_list:
        tp = os.path.join(p, file_name)
        if os.path.exists(tp):
            return tp


def programUpdateProc(self, repo_path):
    from subprocess import Popen
    import os, sys
    from PythonQt import QtCore
    user = self.username
    if not user: user = ''
    password = self.password
    if not password: password = ''
    app_path = QtCore.QCoreApplication.applicationFilePath()
    try:
        if sys.platform[:3] == 'win':
            python_path = which('python.exe')
            repo_path = repo_path.encode('1251')
            app_path = app_path.encode('1251')
        else:
            python_path = os.path.join(os.environ['PYTHON_PATH'], 'python')
        param_list = [python_path, 'svnupdater/appupdater.py', app_path, str(os.getpid()), repo_path, user, password, ]
        p = Popen([l.decode() if type(l)==bytes else l for l in param_list])
        if sys.platform.startswith('linux'):
            p.wait()
        QtCore.QCoreApplication.instance().quit()
    except:
        msgError('python not found')

try:
    import pysvn
    from svnupdater.updater import Updater, InstallerUpdater
    from settings import userSettings

    client = pysvn.Client()
    from PythonQt import QtCore
    import sys,os

    program_dir = QtCore.QDir(QtCore.QCoreApplication.applicationDirPath())
    if sys.platform[:3] == 'win':
        if program_dir.exists("maintenancetool.exe"):
            programUpdater = InstallerUpdater(program_dir.absoluteFilePath("maintenancetool.exe"))
    repo_dir = None
    if programUpdater is None:
        while program_dir.cdUp():
            try:
                client.info(program_dir.absolutePath())
                repo_dir = QtCore.QDir(program_dir)
            except:
                break
        if repo_dir and repo_dir.exists(client.get_adm_dir()):
            try:
                programUpdater = Updater(repo_dir.absolutePath(), u'Обнаружена новая ревизия программы. Обновить?')
                programUpdater.keepAuth = True
                programUpdater.updateProc = programUpdateProc
            except:
                pass
            del programUpdateProc

    if sys.platform[:3] == 'win':
        TortoiseProc = userSettings.value('TortoiseProc')
        if TortoiseProc is None or not os.path.exists(TortoiseProc):
            TortoiseProc = which('TortoiseProc.exe')
    if sys.platform[:5] == 'linux':
        kdesvn = which('kdesvn')

    try:
        from RoadCentre import project
        projectUpdater = Updater(project.baseDir, u'Обнаружена новая ревизия проекта. Обновить?')
        projectUpdater.updateProc = projectUpdateProc
        del projectUpdateProc, project
    except: pass

    del program_dir, repo_dir
    del QtCore, client
    del Updater
    noPySvn = False
except:
    import sys, traceback

    e = sys.exc_info()
    msgError(u'Ошибка инициализации системы обновлений:' + ''.join(traceback.format_tb(e[2])) + '\n' + str(e[1]))
